@@ -15,7 +15,7 @@
  */
 
 
-$min_db_version = 1843;  # update (at least) before a release with the latest function numbrer in upgrade.php
+$min_db_version = 1844;  # update (at least) before a release with the latest function numbrer in upgrade.php
 
 /**
  * check_session
@@ -103,7 +103,8 @@ function init_session($username, $is_admin = false) {
     $_SESSION['sessid']['roles'] = array();
     $_SESSION['sessid']['roles'][] = $is_admin ? 'admin' : 'user';
     $_SESSION['sessid']['username'] = $username;
-    $_SESSION['PFA_token'] = md5(uniqid("", true));
+
+    $_SESSION['PFA_token'] = md5(random_bytes(8) . uniqid('pfa', true));
 
     return $status;
 }
@@ -399,6 +400,10 @@ function safeserver($param, $default = "") {
     if (isset($_SERVER[$param])) {
         $retval = $_SERVER[$param];
     }
+
+    if (!is_string($retval)) {
+        throw new \InvalidArgumentException("\%_SERVER should only contain string value(s)");
+    }
     return $retval;
 }
 
@@ -411,7 +416,7 @@ function safeserver($param, $default = "") {
  */
 function safecookie($param, $default = "") {
     $retval = $default;
-    if (isset($_COOKIE[$param]) && is_string($_COOKIE[$param])) {
+    if (isset($_COOKIE[$param])) {
         $retval = $_COOKIE[$param];
     }
     return $retval;
@@ -638,7 +643,7 @@ function check_owner($username, $domain) {
 
 /**
  * List domains for an admin user.
- * @param String $username
+ * @param string $username
  * @return array of domain names.
  */
 function list_domains_for_admin($username) {
@@ -887,7 +892,7 @@ function validate_password($password) {
             }
             continue;
         }
-       
+
         if (!preg_match($regex, $password)) {
             $msgparts = preg_split("/ /", $message, 2);
             if (count($msgparts) == 1) {
@@ -925,8 +930,10 @@ function _pacrypt_crypt($pw, $pw_db = '') {
     if ($pw_db) {
         return crypt($pw, $pw_db);
     }
-    // Throws E_NOTICE as salt is not specified.
-    return crypt($pw);
+    // PHP8 - we have to specify a salt here....
+    $salt = substr(str_shuffle('0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'), 0, 2);
+
+    return crypt($pw, $salt);
 }
 
 /**
@@ -1428,19 +1435,20 @@ function to64($v, $n) {
  * Action: Send email
  * Call: smtp_mail (string to, string from, string subject, string body]) - or -
  * Call: smtp_mail (string to, string from, string data) - DEPRECATED
- * @param String - To:
- * @param String - From:
- * @param String - Subject: (if called with 4 parameters) or full mail body (if called with 3 parameters)
- * @param String (optional) - Password
- * @param String (optional, but recommended) - mail body
+ * @param string $to
+ * @param string $from
+ * @param string $subject  (if called with 4 parameters) or full mail body (if called with 3 parameters)
+ * @param string $password (optional) - Password
+ * @param string $body (optional, but recommended) - mail body
  * @return bool - true on success, otherwise false
  * TODO: Replace this with something decent like PEAR::Mail or Zend_Mail.
  */
 function smtp_mail($to, $from, $data, $password = "", $body = "") {
     global $CONF;
+
     $smtpd_server = $CONF['smtp_server'];
     $smtpd_port = $CONF['smtp_port'];
-    //$smtp_server = $_SERVER["SERVER_NAME"];
+
     $smtp_server = php_uname('n');
     if (!empty($CONF['smtp_client'])) {
         $smtp_server = $CONF['smtp_client'];
@@ -1559,6 +1567,54 @@ $DEBUG_TEXT = <<<EOF
     </ul>
 EOF;
 
+
+/**
+ * @return string - PDO DSN for PHP.
+ * @throws Exception
+ */
+function db_connection_string() {
+    global $CONF;
+    $dsn = null;
+    if (db_mysql()) {
+        $socket = false;
+        if (Config::has('database_socket')) {
+            $socket = Config::read_string('database_socket');
+        }
+
+        $database_name = Config::read_string('database_name');
+
+        $dsn = 'mysql:';
+        if ($socket) {
+            $dsn .= "unix_socket={$socket}";
+        } else {
+            $dsn .= "host={$CONF['database_host']}";
+        }
+
+        if (isset($CONF['database_port'])) {
+            $dsn .= ";port={$CONF['database_port']}";
+        }
+
+        $dsn .= ";dbname={$database_name};charset=UTF8";
+    } elseif (db_sqlite()) {
+        $db = $CONF['database_name'];
+
+        $dsn = "sqlite:{$db}";
+    } elseif (db_pgsql()) {
+        $dsn = "pgsql:dbname={$CONF['database_name']}";
+        if (isset($CONF['database_host'])) {
+            $dsn .= ";host={$CONF['database_host']}";
+        }
+        if (isset($CONF['database_port'])) {
+            $dsn .= ";port={$CONF['database_port']}";
+        }
+        $dsn .= ";options='-c client_encoding=utf8'";
+    } else {
+        throw new Exception("<p style='color: red'>FATAL Error:<br />Invalid \$CONF['database_type'] <br/>'pgsql', 'mysql' or 'sqlite' supported. <br/> Please fix your config.inc.php!</p>");
+    }
+
+    return $dsn;
+}
+
 /**
  * db_connect
  * Action: Makes a connection to the database if it doesn't exist
@@ -1579,28 +1635,19 @@ function db_connect() {
 
     $link = false;
 
+    // throws.
+    $dsn = db_connection_string();
+
     $options = array(
         PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
+        PDO::ATTR_TIMEOUT => 5,
     );
     $username_password = true;
 
     $queries = array();
 
-    $dsn = null;
 
     if (db_mysql()) {
-        $socket = false;
-        if (Config::has('database_socket')) {
-            $socket = Config::read_string('database_socket');
-        }
-
-        $database_name = Config::read_string('database_name');
-
-        if ($socket) {
-            $dsn = "mysql:unix_socket={$socket};dbname={$database_name};charset=UTF8";
-        } else {
-            $dsn = "mysql:host={$CONF['database_host']};dbname={$database_name};charset=UTF8";
-        }
         if (Config::bool('database_use_ssl')) {
             $options[PDO::MYSQL_ATTR_SSL_KEY] = Config::read_string('database_ssl_key');
             $options[PDO::MYSQL_ATTR_SSL_CA] = Config::read_string('database_ssl_ca');
@@ -1622,31 +1669,23 @@ function db_connect() {
         $db = $CONF['database_name'];
 
         if (!file_exists($db)) {
-            $error_text = 'SQLite database missing: '. $db;
+            $error_text = 'SQLite database missing: ' . $db;
             throw new Exception($error_text);
         }
 
         if (!is_writeable($db)) {
-            $error_text = 'SQLite database not writeable: '. $db;
+            $error_text = 'SQLite database not writeable: ' . $db;
             throw new Exception($error_text);
         }
 
         if (!is_writeable(dirname($db))) {
-            $error_text = 'The directory the SQLite database is in is not writeable: '. dirname($db);
+            $error_text = 'The directory the SQLite database is in is not writeable: ' . dirname($db);
             throw new Exception($error_text);
         }
 
-        $dsn = "sqlite:{$db}";
         $username_password = false;
     } elseif (db_pgsql()) {
-        $dsn = "pgsql:dbname={$CONF['database_name']}";
-        if (isset($CONF['database_host'])) {
-            $dsn .= ";host={$CONF['database_host']}";
-        }
-        if (isset($CONF['database_port'])) {
-            $dsn .= ";port={$CONF['database_port']}";
-        }
-        $dsn .= ";options='-c client_encoding=utf8'";
+        // nothing to do.
     } else {
         throw new Exception("<p style='color: red'>FATAL Error:<br />Invalid \$CONF['database_type']! Please fix your config.inc.php!</p>");
     }
@@ -1920,14 +1959,19 @@ function db_insert($table, array $values, $timestamp = array('created', 'modifie
  * @param array $timestamp (optional) - array of fields to set to now() - default: array('modified')
  * @return int - number of updated rows
  */
-function db_update($table, $where_col, $where_value, $values, $timestamp = array('modified'), $throw_exceptions = false) {
+function db_update(string $table, string $where_col, string $where_value, array $values, array $timestamp = array('modified'), bool $throw_exceptions = false): int {
     $table_key = table_by_key($table);
 
-    $sql = "UPDATE $table_key SET ";
-
     $pvalues = array();
 
     $set = array();
+
+    foreach ($timestamp as $k) {
+        if (!isset($values[$k])) {
+            $values[$k] = 'x'; // timestamp field not in the values list, add it in so we set it to now() see #469
+        }
+    }
+
     foreach ($values as $key => $value) {
         if (in_array($key, $timestamp)) {
             if (db_sqlite()) {
@@ -1935,10 +1979,11 @@ function db_update($table, $where_col, $where_value, $values, $timestamp = array
             } else {
                 $set[] = " $key = now() ";
             }
-        } else {
-            $set[] = " $key = :$key ";
-            $pvalues[$key] = $value;
+            continue;
         }
+
+        $set[] = " $key = :$key ";
+        $pvalues[$key] = $value;
     }
 
     $pvalues['where'] = $where_value;
@@ -2273,7 +2318,7 @@ function gen_show_status($show_alias) {
             }
         }
     } else {
-        $stat_string .= ";&nbsp;";
+        $stat_string .= "&nbsp;";
     }
 
     //   $stat_string .= "<span style='background-color:green'> &nbsp; </span> &nbsp;" .
