@@ -1,10 +1,9 @@
 <?php
+
 # $Id$
 
 /**
  * Handlers User level alias actions - e.g. add alias, get aliases, update etc.
- * @property $username name of alias
- * @property $return return of methods
  */
 class AliasHandler extends PFAHandler {
     protected $db_table = 'alias';
@@ -23,14 +22,14 @@ class AliasHandler extends PFAHandler {
         # (for existing aliases, init() hides it for non-mailbox aliases)
         $mbgoto = 1 - $this->new;
 
-        $this->struct=array(
+        $this->struct = array(
             # field name                allow       display in...   type    $PALANG label                     $PALANG description                 default / ...
             #                           editing?    form    list
-            'status'           => pacol(0,          0,      0,      'html', ''                              , ''                                , '', '',
+            'status'           => pacol(0,          0,      0,      'html', ''                              , ''                                , '', array(),
                 array('not_in_db' => 1)  ),
             'address'          => pacol($this->new, 1,      1,      'mail', 'alias'                         , 'pCreate_alias_catchall_text'     ),
             'localpart'        => pacol($this->new, 0,      0,      'text', 'alias'                         , 'pCreate_alias_catchall_text'     , '',
-                /*options*/ '',
+                /*options*/ array(),
                 /*not_in_db*/ 1                         ),
             'domain'           => pacol($this->new, 0,      1,      'enum', ''                              , ''                                , '',
                 /*options*/ $this->allowed_domains      ),
@@ -38,24 +37,24 @@ class AliasHandler extends PFAHandler {
             'is_mailbox'       => pacol(0,          0,      1,      'int', ''                             , ''                                , 0 ,
                 # technically 'is_mailbox' is bool, but the automatic bool conversion breaks the query. Flagging it as int avoids this problem.
                 # Maybe having a vbool type (without the automatic conversion) would be cleaner - we'll see if we need it.
-                /*options*/ '',
+                /*options*/ array(),
                 /*not_in_db*/ 0,
                 /*dont_write_to_db*/ 1,
                 /*select*/ 'coalesce(__is_mailbox,0) as is_mailbox' ),
                 /*extrafrom set via set_is_mailbox_extrafrom() */
             '__mailbox_username' => pacol( 0,       0,      1,      'vtxt', ''                              , ''                                , 0),  # filled via is_mailbox
             'goto_mailbox'     => pacol($mbgoto,    $mbgoto,$mbgoto,'bool', 'pEdit_alias_forward_and_store' , ''                                , 0,
-                /*options*/ '',
+                /*options*/ array(),
                 /*not_in_db*/ 1                         ), # read_from_db_postprocess() sets the value
             'on_vacation'      => pacol(1,          0,      1,      'bool', 'pUsersMenu_vacation'           , ''                                , 0 ,
-                /*options*/ '',
+                /*options*/ array(),
                 /*not_in_db*/ 1                         ), # read_from_db_postprocess() sets the value - TODO: read active flag from vacation table instead?
             'created'          => pacol(0,          0,      0,      'ts',   'created'                       , ''                                ),
             'modified'         => pacol(0,          0,      1,      'ts',   'last_modified'                 , ''                                ),
             'active'           => pacol(1,          1,      1,      'bool', 'active'                        , ''                                , 1     ),
-            '_can_edit'        => pacol(0,          0,      1,      'vnum', ''                              , ''                                , 0 , '',
+            '_can_edit'        => pacol(0,          0,      1,      'vnum', ''                              , ''                                , 0 , array(),
                 array('select' => '1 as _can_edit')  ),
-            '_can_delete'      => pacol(0,          0,      1,      'vnum', ''                              , ''                                , 0 , '',
+            '_can_delete'      => pacol(0,          0,      1,      'vnum', ''                              , ''                                , 0 , array(),
                 array('select' => '1 as _can_delete')  ), # read_from_db_postprocess() updates the value
                 # aliases listed in $CONF[default_aliases] are read-only for domain admins if $CONF[special_alias_control] is NO.
         );
@@ -138,11 +137,14 @@ class AliasHandler extends PFAHandler {
      * AliasHandler needs some special handling in init() and therefore overloads the function.
      * It also calls parent::init()
      */
-    public function init($id) {
-        @list($local_part, $domain) = explode('@', $id); # supress error message if $id doesn't contain '@'
-
-        if ($local_part == '*') { # catchall - postfix expects '@domain', not '*@domain'
-            $id = '@' . $domain;
+    public function init(string $id): bool {
+        $bits = explode('@', $id);
+        if (sizeof($bits) == 2) {
+            $local_part = $bits[0];
+            $domain = $bits[1];
+            if ($local_part == '*') { # catchall - postfix expects '@domain', not '*@domain'
+                $id = '@' . $domain;
+            }
         }
 
         $retval = parent::init($id);
@@ -150,7 +152,7 @@ class AliasHandler extends PFAHandler {
         if (!$retval) {
             return false;
         } # parent::init() failed, no need to continue
-        
+
         # hide 'goto_mailbox' for non-mailbox aliases
         # parent::init called view() before, so we can rely on having $this->result filled
         # (only validate_new_id() is called from parent::init and could in theory change $this->result)
@@ -190,7 +192,7 @@ class AliasHandler extends PFAHandler {
             $this->errormsg[$this->id_field] = Config::lang('pCreate_alias_address_text_error3');
             return false;
         }
- 
+
         # TODO: already checked in set() - does it make sense to check it here also? Only advantage: it's an early check
         #        if (!in_array($domain, $this->allowed_domains)) {
         #            $this->errormsg[] = Config::lang('pCreate_alias_address_text_error1');
@@ -253,7 +255,7 @@ class AliasHandler extends PFAHandler {
         }
     }
 
-    protected function setmore($values) {
+    protected function setmore(array $values) {
         if ($this->new) {
             if ($this->struct['address']['display_in_form'] == 1) { # default mode - split off 'domain' field from 'address' # TODO: do this unconditional?
                 list(/*NULL*/, $domain) = explode('@', $values['address']);
@@ -302,7 +304,7 @@ class AliasHandler extends PFAHandler {
         $this->values['goto'] = join(',', $values['goto']);
     }
 
-    protected function storemore() {
+    protected function postSave(): bool {
         # TODO: if alias belongs to a mailbox, update mailbox active status
         return true;
     }
@@ -310,7 +312,10 @@ class AliasHandler extends PFAHandler {
     protected function read_from_db_postprocess($db_result) {
         foreach ($db_result as $key => $value) {
             # split comma-separated 'goto' into an array
-            $db_result[$key]['goto'] = explode(',', $db_result[$key]['goto']);
+            $goto = $db_result[$key]['goto'] ?? null;
+            if (is_string($goto)) {
+                $db_result[$key]['goto'] = explode(',', $goto);
+            }
 
             # Vacation enabled?
             list($db_result[$key]['on_vacation'], $db_result[$key]['goto']) = remove_from_array($db_result[$key]['goto'], $this->getVacationAlias());
@@ -326,12 +331,12 @@ class AliasHandler extends PFAHandler {
 
             # editing a default alias (postmaster@ etc.) is only allowed if special_alias_control is allowed or if the user is a superadmin
             $tmp = preg_split('/\@/', $db_result[$key]['address']);
-            if (!$this->is_superadmin && !Config::bool('special_alias_control') && array_key_exists($tmp[0], Config::Read('default_aliases'))) {
+            if (!$this->is_superadmin && !Config::bool('special_alias_control') && array_key_exists($tmp[0], Config::read_array('default_aliases'))) {
                 $db_result[$key]['_can_edit'] = 0;
                 $db_result[$key]['_can_delete'] = 0;
             }
 
-            if ($this->struct['status']['display_in_list'] && Config::Bool('show_status')) {
+            if ($this->struct['status']['display_in_list'] && Config::bool('show_status')) {
                 $db_result[$key]['status'] = gen_show_status($db_result[$key]['address']);
             }
         }
@@ -353,7 +358,7 @@ class AliasHandler extends PFAHandler {
         return array($condition, $searchmode);
     }
 
-    public function getList($condition, $searchmode = array(), $limit=-1, $offset=-1) {
+    public function getList($condition, $searchmode = array(), $limit=-1, $offset=-1): bool {
         list($condition, $searchmode) = $this->condition_ignore_mailboxes($condition, $searchmode);
         $this->set_is_mailbox_extrafrom($condition, $searchmode);
         $result = parent::getList($condition, $searchmode, $limit, $offset);
@@ -397,6 +402,11 @@ class AliasHandler extends PFAHandler {
                     $errors[] = "$singlegoto: $email_check";
                 }
             }
+            if ($this->called_by != "MailboxHandler" && $this->id == $singlegoto) {
+                // The MailboxHandler needs to create an alias that points to itself (for the mailbox)
+                // Otherwise, disallow such aliases as they cause severe trouble in the mail system
+                $errors[] = "$singlegoto: " . Config::Lang('alias_points_to_itself');
+            }
         }
 
         if (count($errors)) {
@@ -436,15 +446,19 @@ class AliasHandler extends PFAHandler {
     * Returns the vacation alias for this user.
     * i.e. if this user's username was roger@example.com, and the autoreply domain was set to
     * autoreply.fish.net in config.inc.php we'd return roger#example.com@autoreply.fish.net
+    *
     * @return string an email alias.
     */
     protected function getVacationAlias() {
-        $vacation_goto = str_replace('@', '#', $this->id);
-        return $vacation_goto . '@' . Config::read('vacation_domain');
+        if ($this->id !== null) {
+            $vacation_goto = str_replace('@', '#', $this->id);
+            return $vacation_goto . '@' . Config::read_string('vacation_domain');
+        }
+        return "unknown@" . Config::read_string('vacation_domain');
     }
- 
+
     /**
-     *  @return true on success false on failure
+     *  @return boolean
      */
     public function delete() {
         if (! $this->view()) {
